@@ -152,23 +152,24 @@ func TestGlobalRuleClient(t *testing.T) {
 	closedCh := make(chan struct{})
 	close(closedCh)
 	cli := newGlobalRuleClient(&cluster{
-		baseURL:          u.String(),
-		cli:              http.DefaultClient,
-		cache:            &dummyCache{},
-		cacheSynced:      closedCh,
-		metricsCollector: metrics.NewPrometheusCollector(),
+		baseURL:           u.String(),
+		cli:               http.DefaultClient,
+		cache:             &dummyCache{},
+		generatedObjCache: &dummyCache{},
+		cacheSynced:       closedCh,
+		metricsCollector:  metrics.NewPrometheusCollector(),
 	})
 
 	// Create
 	obj, err := cli.Create(context.Background(), &v1.GlobalRule{
 		ID: "1",
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, obj.ID, "1")
 
 	obj, err = cli.Create(context.Background(), &v1.GlobalRule{
 		ID: "2",
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, obj.ID, "2")
 
@@ -192,7 +193,7 @@ func TestGlobalRuleClient(t *testing.T) {
 		Plugins: map[string]interface{}{
 			"prometheus": struct{}{},
 		},
-	})
+	}, false)
 	assert.Nil(t, err)
 	objs, err = cli.List(context.Background())
 	assert.Nil(t, err)
