@@ -152,11 +152,12 @@ func TestStreamRouteClient(t *testing.T) {
 	closedCh := make(chan struct{})
 	close(closedCh)
 	cli := newStreamRouteClient(&cluster{
-		baseURL:          u.String(),
-		cli:              http.DefaultClient,
-		cache:            &dummyCache{},
-		cacheSynced:      closedCh,
-		metricsCollector: metrics.NewPrometheusCollector(),
+		baseURL:           u.String(),
+		cli:               http.DefaultClient,
+		cache:             &dummyCache{},
+		generatedObjCache: &dummyCache{},
+		cacheSynced:       closedCh,
+		metricsCollector:  metrics.NewPrometheusCollector(),
 	})
 
 	// Create
@@ -165,7 +166,7 @@ func TestStreamRouteClient(t *testing.T) {
 		ServerPort: 8001,
 		UpstreamId: "1",
 		SNI:        "a.test.com",
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, obj.ID, "1")
 	assert.Equal(t, obj.SNI, "a.test.com")
@@ -175,7 +176,7 @@ func TestStreamRouteClient(t *testing.T) {
 		ServerPort: 8002,
 		UpstreamId: "1",
 		SNI:        "*.test.com",
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, obj.ID, "2")
 	assert.Equal(t, obj.SNI, "*.test.com")
@@ -198,7 +199,7 @@ func TestStreamRouteClient(t *testing.T) {
 	_, err = cli.Update(context.Background(), &v1.StreamRoute{
 		ID:         "2",
 		UpstreamId: "112",
-	})
+	}, false)
 	assert.Nil(t, err)
 	objs, err = cli.List(context.Background())
 	assert.Nil(t, err)
