@@ -153,6 +153,7 @@ func TestUpstreamClient(t *testing.T) {
 		baseURL:                 u.String(),
 		cli:                     http.DefaultClient,
 		cache:                   &dummyCache{},
+		generatedObjCache:       &dummyCache{},
 		cacheSynced:             closedCh,
 		metricsCollector:        metrics.NewPrometheusCollector(),
 		upstreamServiceRelation: &dummyUpstreamServiceRelation{},
@@ -181,7 +182,7 @@ func TestUpstreamClient(t *testing.T) {
 		Type:  lbType,
 		Key:   key,
 		Nodes: nodes,
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, "1", obj.ID)
 
@@ -194,7 +195,7 @@ func TestUpstreamClient(t *testing.T) {
 		Type:  lbType,
 		Key:   key,
 		Nodes: nodes,
-	})
+	}, false)
 	assert.Nil(t, err)
 	assert.Equal(t, "2", obj.ID)
 
@@ -221,7 +222,7 @@ func TestUpstreamClient(t *testing.T) {
 		Type:  "chash",
 		Key:   key,
 		Nodes: nodes,
-	})
+	}, false)
 	assert.Nil(t, err)
 	objs, err = cli.List(context.Background())
 	assert.Nil(t, err)
