@@ -17,10 +17,9 @@ package kube
 import (
 	"errors"
 
-	extensionsv1beta1 "k8s.io/api/extensions/v1beta1"
 	networkingv1 "k8s.io/api/networking/v1"
 	networkingv1beta1 "k8s.io/api/networking/v1beta1"
-	listersextensionsv1beta1 "k8s.io/client-go/listers/extensions/v1beta1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	listersnetworkingv1 "k8s.io/client-go/listers/networking/v1"
 	listersnetworkingv1beta1 "k8s.io/client-go/listers/networking/v1beta1"
 )
@@ -30,8 +29,6 @@ const (
 	IngressV1 = "networking/v1"
 	// IngressV1beta1 represents the Ingress in networking/v1beta1 group version.
 	IngressV1beta1 = "networking/v1beta1"
-	// IngressExtensionsV1beta1 represents the Ingress in extensions/v1beta1 group version.
-	IngressExtensionsV1beta1 = "extensions/v1beta1"
 )
 
 // IngressLister is an encapsulation for the lister of Kubernetes
@@ -41,8 +38,6 @@ type IngressLister interface {
 	V1(string, string) (Ingress, error)
 	// V1beta1 gets the ingress in networking/v1beta1.
 	V1beta1(string, string) (Ingress, error)
-	// ExtensionsV1beta1 gets the ingress in extensions/v1beta1.
-	ExtensionsV1beta1(string, string) (Ingress, error)
 }
 
 // IngressInformer is an encapsulation for the informer of Kubernetes
@@ -63,12 +58,11 @@ type Ingress interface {
 	// V1beta1 returns the ingress in networking/v1beta1, the real
 	// ingress must be in networking/v1beta1, or V1beta1() will panic.
 	V1beta1() *networkingv1beta1.Ingress
-	// ExtensionsV1beta1 returns the ingress in extensions/v1beta1, the real
-	// ingress must be in extensions/v1beta1, or ExtensionsV1beta1() will panic.
-	ExtensionsV1beta1() *extensionsv1beta1.Ingress
 	// ResourceVersion returns the the resource version field inside
 	// the real Ingress.
 	ResourceVersion() string
+
+	metav1.Object
 }
 
 // IngressEvents contains the ingress key (namespace/name)
@@ -80,10 +74,10 @@ type IngressEvent struct {
 }
 
 type ingress struct {
-	groupVersion      string
-	v1                *networkingv1.Ingress
-	v1beta1           *networkingv1beta1.Ingress
-	extensionsV1beta1 *extensionsv1beta1.Ingress
+	groupVersion string
+	v1           *networkingv1.Ingress
+	v1beta1      *networkingv1beta1.Ingress
+	metav1.Object
 }
 
 func (ing *ingress) V1() *networkingv1.Ingress {
@@ -100,31 +94,20 @@ func (ing *ingress) V1beta1() *networkingv1beta1.Ingress {
 	return ing.v1beta1
 }
 
-func (ing *ingress) ExtensionsV1beta1() *extensionsv1beta1.Ingress {
-	if ing.groupVersion != IngressExtensionsV1beta1 {
-		panic("not a extensions/v1beta1 ingress")
-	}
-	return ing.extensionsV1beta1
-}
-
 func (ing *ingress) GroupVersion() string {
 	return ing.groupVersion
 }
 
 func (ing *ingress) ResourceVersion() string {
-	if ing.GroupVersion() == IngressV1 {
-		return ing.V1().ResourceVersion
-	}
 	if ing.GroupVersion() == IngressV1beta1 {
 		return ing.V1beta1().ResourceVersion
 	}
-	return ing.ExtensionsV1beta1().ResourceVersion
+	return ing.V1().ResourceVersion
 }
 
 type ingressLister struct {
-	v1Lister                listersnetworkingv1.IngressLister
-	v1beta1Lister           listersnetworkingv1beta1.IngressLister
-	extensionsV1beta1Lister listersextensionsv1beta1.IngressLister
+	v1Lister      listersnetworkingv1.IngressLister
+	v1beta1Lister listersnetworkingv1beta1.IngressLister
 }
 
 func (l *ingressLister) V1(namespace, name string) (Ingress, error) {
@@ -135,6 +118,7 @@ func (l *ingressLister) V1(namespace, name string) (Ingress, error) {
 	return &ingress{
 		groupVersion: IngressV1,
 		v1:           ing,
+		Object:       ing,
 	}, nil
 }
 
@@ -146,17 +130,7 @@ func (l *ingressLister) V1beta1(namespace, name string) (Ingress, error) {
 	return &ingress{
 		groupVersion: IngressV1beta1,
 		v1beta1:      ing,
-	}, nil
-}
-
-func (l *ingressLister) ExtensionsV1beta1(namespace, name string) (Ingress, error) {
-	ing, err := l.extensionsV1beta1Lister.Ingresses(namespace).Get(name)
-	if err != nil {
-		return nil, err
-	}
-	return &ingress{
-		groupVersion:      IngressExtensionsV1beta1,
-		extensionsV1beta1: ing,
+		Object:       ing,
 	}, nil
 }
 
@@ -168,16 +142,13 @@ func MustNewIngress(obj interface{}) Ingress {
 		return &ingress{
 			groupVersion: IngressV1,
 			v1:           ing,
+			Object:       ing,
 		}
 	case *networkingv1beta1.Ingress:
 		return &ingress{
 			groupVersion: IngressV1beta1,
 			v1beta1:      ing,
-		}
-	case *extensionsv1beta1.Ingress:
-		return &ingress{
-			groupVersion:      IngressExtensionsV1beta1,
-			extensionsV1beta1: ing,
+			Object:       ing,
 		}
 	default:
 		panic("invalid ingress type")
@@ -193,16 +164,13 @@ func NewIngress(obj interface{}) (Ingress, error) {
 		return &ingress{
 			groupVersion: IngressV1,
 			v1:           ing,
+			Object:       ing,
 		}, nil
 	case *networkingv1beta1.Ingress:
 		return &ingress{
 			groupVersion: IngressV1beta1,
 			v1beta1:      ing,
-		}, nil
-	case *extensionsv1beta1.Ingress:
-		return &ingress{
-			groupVersion:      IngressExtensionsV1beta1,
-			extensionsV1beta1: ing,
+			Object:       ing,
 		}, nil
 	default:
 		return nil, errors.New("invalid ingress type")
@@ -210,11 +178,9 @@ func NewIngress(obj interface{}) (Ingress, error) {
 }
 
 // NewIngressLister creates an version-neutral Ingress lister.
-func NewIngressLister(v1 listersnetworkingv1.IngressLister, v1beta1 listersnetworkingv1beta1.IngressLister,
-	extensionsv1beta1 listersextensionsv1beta1.IngressLister) IngressLister {
+func NewIngressLister(v1 listersnetworkingv1.IngressLister, v1beta1 listersnetworkingv1beta1.IngressLister) IngressLister {
 	return &ingressLister{
-		v1Lister:                v1,
-		v1beta1Lister:           v1beta1,
-		extensionsV1beta1Lister: extensionsv1beta1,
+		v1Lister:      v1,
+		v1beta1Lister: v1beta1,
 	}
 }
