@@ -97,20 +97,26 @@ func apisixRequestTestHandler(t *testing.T, metrics []*io_prometheus_client.Metr
 		assert.Len(t, m, 2)
 
 		assert.Equal(t, *m[0].Counter.Value, float64(2))
+		assert.Len(t, m[0].Label, 4)
 		assert.Equal(t, *m[0].Label[0].Name, "controller_namespace")
 		assert.Equal(t, *m[0].Label[0].Value, "default")
 		assert.Equal(t, *m[0].Label[1].Name, "controller_pod")
 		assert.Equal(t, *m[0].Label[1].Value, "")
-		assert.Equal(t, *m[0].Label[2].Name, "resource")
-		assert.Equal(t, *m[0].Label[2].Value, "route")
+		assert.Equal(t, *m[0].Label[2].Name, "op")
+		assert.Equal(t, *m[0].Label[2].Value, "read")
+		assert.Equal(t, *m[0].Label[3].Name, "resource")
+		assert.Equal(t, *m[0].Label[3].Value, "route")
 
 		assert.Equal(t, *m[1].Counter.Value, float64(1))
+		assert.Len(t, m[1].Label, 4)
 		assert.Equal(t, *m[1].Label[0].Name, "controller_namespace")
 		assert.Equal(t, *m[1].Label[0].Value, "default")
 		assert.Equal(t, *m[1].Label[1].Name, "controller_pod")
 		assert.Equal(t, *m[1].Label[1].Value, "")
-		assert.Equal(t, *m[1].Label[2].Name, "resource")
-		assert.Equal(t, *m[1].Label[2].Value, "upstream")
+		assert.Equal(t, *m[1].Label[2].Name, "op")
+		assert.Equal(t, *m[1].Label[2].Value, "read")
+		assert.Equal(t, *m[1].Label[3].Name, "resource")
+		assert.Equal(t, *m[1].Label[3].Value, "upstream")
 	}
 }
 
@@ -206,9 +212,9 @@ func TestPrometheusCollector(t *testing.T) {
 	c.RecordAPISIXCode(404, "route")
 	c.RecordAPISIXCode(500, "upstream")
 	c.RecordAPISIXLatency(500*time.Millisecond, "create")
-	c.IncrAPISIXRequest("route")
-	c.IncrAPISIXRequest("route")
-	c.IncrAPISIXRequest("upstream")
+	c.IncrAPISIXReadRequest("route")
+	c.IncrAPISIXReadRequest("route")
+	c.IncrAPISIXReadRequest("upstream")
 	c.IncrCheckClusterHealth("test")
 	c.IncrSyncOperation("schema", "failure")
 	c.IncrSyncOperation("endpoint", "success")
