@@ -23,9 +23,9 @@ import (
 	"strings"
 
 	"go.uber.org/zap"
-	extensionsv1beta1 "k8s.io/api/extensions/v1beta1"
 	networkingv1 "k8s.io/api/networking/v1"
 	networkingv1beta1 "k8s.io/api/networking/v1beta1"
+	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	listerscorev1 "k8s.io/client-go/listers/core/v1"
@@ -65,6 +65,9 @@ type IngressTranslator interface {
 	TranslateOldIngress(kube.Ingress) (*translation.TranslateContext, error)
 	// TranslateSSLV2 translate networkingv1.IngressTLS to APISIX SSL
 	TranslateIngressTLS(namespace, ingName, secretName string, hosts []string) (*apisixv1.Ssl, error)
+	// TranslateIngressDeleteEvent composes a couple of APISIX Routes and upstreams according
+	// to the given Ingress resource.
+	TranslateIngressDeleteEvent(ing kube.Ingress, args ...bool) (*translation.TranslateContext, error)
 }
 
 func NewIngressTranslator(opts *TranslatorOptions,
@@ -112,8 +115,17 @@ func (t *translator) TranslateIngress(ing kube.Ingress, args ...bool) (*translat
 		return t.translateIngressV1(ing.V1(), skipVerify)
 	case kube.IngressV1beta1:
 		return t.translateIngressV1beta1(ing.V1beta1(), skipVerify)
-	case kube.IngressExtensionsV1beta1:
-		return t.translateIngressExtensionsV1beta1(ing.ExtensionsV1beta1(), skipVerify)
+	default:
+		return nil, fmt.Errorf("translator: source group version not supported: %s", ing.GroupVersion())
+	}
+}
+
+func (t *translator) TranslateIngressDeleteEvent(ing kube.Ingress, args ...bool) (*translation.TranslateContext, error) {
+	switch ing.GroupVersion() {
+	case kube.IngressV1:
+		return t.translateOldIngressV1(ing.V1())
+	case kube.IngressV1beta1:
+		return t.translateOldIngressV1beta1(ing.V1beta1())
 	default:
 		return nil, fmt.Errorf("translator: source group version not supported: %s", ing.GroupVersion())
 	}
@@ -144,6 +156,9 @@ func (t *translator) translateIngressV1(ing *networkingv1.Ingress, skipVerify bo
 		ns = ingress.ServiceNamespace
 	}
 	for _, rule := range ing.Spec.Rules {
+		if rule.HTTP == nil {
+			continue
+		}
 		for _, pathRule := range rule.HTTP.Paths {
 			var (
 				ups *apisixv1.Upstream
@@ -387,113 +402,6 @@ func (t *translator) translateUpstreamFromIngressV1(namespace string, backend *n
 	return ups, nil
 }
 
-func (t *translator) translateIngressExtensionsV1beta1(ing *extensionsv1beta1.Ingress, skipVerify bool) (*translation.TranslateContext, error) {
-	ctx := translation.DefaultEmptyTranslateContext()
-	ingress := t.TranslateAnnotations(ing.Annotations)
-
-	// add https
-	for _, tls := range ing.Spec.TLS {
-		ssl, err := t.TranslateIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
-		if err != nil {
-			log.Errorw("failed to translate ingress tls to apisix tls",
-				zap.Error(err),
-				zap.Any("ingress", ing),
-			)
-			return nil, err
-		}
-		ctx.AddSSL(ssl)
-	}
-	ns := ing.Namespace
-	if ingress.ServiceNamespace != "" {
-		ns = ingress.ServiceNamespace
-	}
-	for _, rule := range ing.Spec.Rules {
-		for _, pathRule := range rule.HTTP.Paths {
-			var (
-				ups *apisixv1.Upstream
-				err error
-			)
-			if pathRule.Backend.ServiceName != "" {
-				// Structure here is same to ingress.extensions/v1beta1, so just use this method.
-				if skipVerify {
-					ups = t.translateDefaultUpstreamFromIngressV1beta1(ns, pathRule.Backend.ServiceName, pathRule.Backend.ServicePort)
-				} else {
-					ups, err = t.translateUpstreamFromIngressV1beta1(ns, pathRule.Backend.ServiceName, pathRule.Backend.ServicePort)
-					if err != nil {
-						log.Errorw("failed to translate ingress backend to upstream",
-							zap.Error(err),
-							zap.Any("ingress", ing),
-						)
-						return nil, err
-					}
-				}
-				if ingress.UpstreamScheme != "" {
-					ups.Scheme = ingress.UpstreamScheme
-				}
-				ctx.AddUpstream(ups)
-			}
-			uris := []string{pathRule.Path}
-			var nginxVars []kubev2.ApisixRouteHTTPMatchExpr
-			if pathRule.PathType != nil {
-				if *pathRule.PathType == extensionsv1beta1.PathTypePrefix {
-					// As per the specification of Ingress path matching rule:
-					// if the last element of the path is a substring of the
-					// last element in request path, it is not a match, e.g. /foo/bar
-					// matches /foo/bar/baz, but does not match /foo/barbaz.
-					// While in APISIX, /foo/bar matches both /foo/bar/baz and
-					// /foo/barbaz.
-					// In order to be conformant with Ingress specification, here
-					// we create two paths here, the first is the path itself
-					// (exact match), the other is path + "/*" (prefix match).
-					prefix := pathRule.Path
-					if strings.HasSuffix(prefix, "/") {
-						prefix += "*"
-					} else {
-						prefix += "/*"
-					}
-					uris = append(uris, prefix)
-				} else if *pathRule.PathType == extensionsv1beta1.PathTypeImplementationSpecific && ingress.UseRegex {
-					nginxVars = append(nginxVars, kubev2.ApisixRouteHTTPMatchExpr{
-						Subject: kubev2.ApisixRouteHTTPMatchExprSubject{
-							Scope: apisixconst.ScopePath,
-						},
-						Op:    apisixconst.OpRegexMatch,
-						Value: &pathRule.Path,
-					})
-					uris = []string{"/*"}
-				}
-			}
-			route := apisixv1.NewDefaultRoute()
-			route.Name = composeIngressRouteName(ing.Namespace, ing.Name, rule.Host, pathRule.Path)
-			route.ID = id.GenID(route.Name)
-			route.Host = rule.Host
-			route.Uris = uris
-			route.EnableWebsocket = ingress.EnableWebSocket
-			if len(nginxVars) > 0 {
-				routeVars, err := t.ApisixTranslator.TranslateRouteMatchExprs(nginxVars)
-				if err != nil {
-					return nil, err
-				}
-				route.Vars = routeVars
-				route.Priority = _regexPriority
-			}
-			if len(ingress.Plugins) > 0 {
-				route.Plugins = *(ingress.Plugins.DeepCopy())
-			}
-
-			if ingress.PluginConfigName != "" {
-				route.PluginConfigId = id.GenID(apisixv1.ComposePluginConfigName(ing.Namespace, ingress.PluginConfigName))
-			}
-
-			if ups != nil {
-				route.UpstreamId = ups.ID
-			}
-			ctx.AddRoute(route)
-		}
-	}
-	return ctx, nil
-}
-
 func (t *translator) translateDefaultUpstreamFromIngressV1beta1(namespace string, svcName string, svcPort intstr.IntOrString) *apisixv1.Upstream {
 	var portNumber int32
 	if svcPort.Type == intstr.String {
@@ -554,20 +462,28 @@ func (t *translator) TranslateOldIngress(ing kube.Ingress) (*translation.Transla
 		return t.translateOldIngressV1(ing.V1())
 	case kube.IngressV1beta1:
 		return t.translateOldIngressV1beta1(ing.V1beta1())
-	case kube.IngressExtensionsV1beta1:
-		return t.translateOldIngressExtensionsv1beta1(ing.ExtensionsV1beta1())
 	default:
 		return nil, fmt.Errorf("translator: source group version not supported: %s", ing.GroupVersion())
 	}
 }
 
+func (t *translator) translateOldIngressTLS(namespace, ingName, secretName string, hosts []string) (*apisixv1.Ssl, error) {
+	ssl, err := t.TranslateIngressTLS(namespace, ingName, secretName, hosts)
+	if err != nil && k8serrors.IsNotFound(err) {
+		return &apisixv1.Ssl{
+			ID: id.GenID(namespace + "_" + fmt.Sprintf("%v-%v", ingName, "tls")),
+		}, nil
+	}
+	return ssl, err
+}
+
 func (t *translator) translateOldIngressV1(ing *networkingv1.Ingress) (*translation.TranslateContext, error) {
 	oldCtx := translation.DefaultEmptyTranslateContext()
 
 	for _, tls := range ing.Spec.TLS {
-		ssl, err := t.TranslateIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
+		ssl, err := t.translateOldIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
 		if err != nil {
-			log.Debugw("failed to translate ingress tls to apisix tls",
+			log.Errorw("failed to translate ingress tls to apisix tls",
 				zap.Error(err),
 				zap.Any("ingress", ing),
 			)
@@ -602,41 +518,12 @@ func (t *translator) translateOldIngressV1beta1(ing *networkingv1beta1.Ingress)
 	oldCtx := translation.DefaultEmptyTranslateContext()
 
 	for _, tls := range ing.Spec.TLS {
-		ssl, err := t.TranslateIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
-		if err != nil {
-			continue
-		}
-		oldCtx.AddSSL(ssl)
-	}
-	for _, rule := range ing.Spec.Rules {
-		for _, pathRule := range rule.HTTP.Paths {
-			name := composeIngressRouteName(ing.Namespace, ing.Name, rule.Host, pathRule.Path)
-			r, err := t.Apisix.Cluster(t.ClusterName).Route().Get(context.Background(), name)
-			if err != nil {
-				continue
-			}
-			if r.UpstreamId != "" {
-				ups := apisixv1.NewDefaultUpstream()
-				ups.ID = r.UpstreamId
-				oldCtx.AddUpstream(ups)
-			}
-			if r.PluginConfigId != "" {
-				pc := apisixv1.NewDefaultPluginConfig()
-				pc.ID = r.PluginConfigId
-				oldCtx.AddPluginConfig(pc)
-			}
-			oldCtx.AddRoute(r)
-		}
-	}
-	return oldCtx, nil
-}
-
-func (t *translator) translateOldIngressExtensionsv1beta1(ing *extensionsv1beta1.Ingress) (*translation.TranslateContext, error) {
-	oldCtx := translation.DefaultEmptyTranslateContext()
-
-	for _, tls := range ing.Spec.TLS {
-		ssl, err := t.TranslateIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
+		ssl, err := t.translateOldIngressTLS(ing.Namespace, ing.Name, tls.SecretName, tls.Hosts)
 		if err != nil {
+			log.Errorw("failed to translate ingress tls to apisix tls",
+				zap.Error(err),
+				zap.Any("ingress", ing),
+			)
 			continue
 		}
 		oldCtx.AddSSL(ssl)
