@@ -50,7 +50,7 @@ ret=0
 
 bash "${SCRIPT_ROOT}"/generate-groups.sh "deepcopy,client,informer,lister" \
   ${PKG_NAME}/pkg/kube/apisix/client ${PKG_NAME}/pkg/kube/apisix/apis \
-  config:v2,v2beta3 ${PKG_NAME} \
+  config:v2 ${PKG_NAME} \
   --output-base "$GENERATED_ROOT" \
   --go-header-file "${SCRIPT_ROOT}"/boilerplate.go.txt \
   --verify-only || ret=$?
@@ -68,7 +68,7 @@ fi
 #if [[ $ret -eq 0 ]]; then
 #bash "${SCRIPT_ROOT}"/generate-groups.sh "register" \
   #${PKG_NAME}/pkg/kube/apisix/apis ${PKG_NAME}/pkg/kube/apisix/apis \
-  #config:v2,v2beta3,v1 ${PKG_NAME} \
+  #config:v2,v1 ${PKG_NAME} \
   #--output-base "$GENERATED_ROOT" \
   #--go-header-file "${SCRIPT_ROOT}"/boilerplate.go.txt \
   #--verify-only|| ret=$?
